/***********************************************************************************************************************
 *
 * blueArgyle - a Java UI for Argyll
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://blueargyle.java.net
 * SCM: https://bitbucket.org/tidalwave/blueargyle-src
 *
 **********************************************************************************************************************/
package it.tidalwave.uniformity.ui.impl.measurement.netbeans;

import javax.annotation.Nonnull;
import java.awt.EventQueue;
import javax.swing.Action;
import javax.swing.JPanel;
import it.tidalwave.swing.SafeActionAdapter;

/***********************************************************************************************************************
 * 
 * @stereotype Presentation
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ControlPanel extends JPanel 
  {
    private final SafeActionAdapter continueAction = new SafeActionAdapter();
    
    private final SafeActionAdapter cancelAction = new SafeActionAdapter();
    
    public ControlPanel()
      {
        assert EventQueue.isDispatchThread();
        initComponents();
        setOpaque(true);
        setBackground(UniformityCheckMeasurementPresentationPanel.CONTROL_PANEL_BACKGROUND);
      }

    public ControlPanel (final @Nonnull Action continueAction, final @Nonnull Action cancelAction)
      {
        this();
        pbMeasurementInProgress.setVisible(false);
        lbInvitation.setVisible(false);
        this.continueAction.bind(continueAction);
        this.cancelAction.bind(cancelAction);
      }

    @Override
    public void removeNotify()
      {
        assert EventQueue.isDispatchThread();
        continueAction.unbind();
        cancelAction.unbind();
        super.removeNotify();
      }
    
    public void setProgressIndicatorVisible (final boolean visible) 
      {
        assert EventQueue.isDispatchThread();
        pbMeasurementInProgress.setVisible(visible);
      }
    
    public void showInvitationToOperateOnTheSensor (final @Nonnull String invitation) 
      {
        assert EventQueue.isDispatchThread();
        lbInvitation.setText(String.format("<html>%s</html>", invitation));
        lbInvitation.setVisible(true);
      }
    
    public void hideInvitationToOperateOnTheSensor()
      {
        lbInvitation.setVisible(false);
      }
    
    /*******************************************************************************************************************
     * 
     * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
     * content of this method is always regenerated by the Form Editor.
     *
     ******************************************************************************************************************/
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        btContinue = new javax.swing.JButton();
        btCancel = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        pbMeasurementInProgress = new javax.swing.JProgressBar();
        lbInvitation = new javax.swing.JLabel();

        setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.name")); // NOI18N

        jLabel1.setForeground(new java.awt.Color(255, 255, 255));
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.jLabel1.text")); // NOI18N
        jLabel1.setVerticalAlignment(javax.swing.SwingConstants.BOTTOM);
        jLabel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(16, 16, 16, 16));
        jLabel1.setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.jLabel1.name")); // NOI18N

        jPanel1.setMaximumSize(new java.awt.Dimension(32767, 48));
        jPanel1.setMinimumSize(new java.awt.Dimension(0, 48));
        jPanel1.setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.jPanel1.name")); // NOI18N
        jPanel1.setOpaque(false);
        jPanel1.setPreferredSize(new java.awt.Dimension(0, 48));
        jPanel1.setSize(new java.awt.Dimension(0, 48));

        btContinue.setAction(continueAction);
        btContinue.setText(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.btContinue.text")); // NOI18N
        btContinue.setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.btContinue.name")); // NOI18N
        btContinue.setPreferredSize(new java.awt.Dimension(102, 32));
        btContinue.setSize(new java.awt.Dimension(102, 32));
        jPanel1.add(btContinue);

        btCancel.setAction(cancelAction);
        btCancel.setText(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.btCancel.text")); // NOI18N
        btCancel.setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.btCancel.name")); // NOI18N
        btCancel.setPreferredSize(new java.awt.Dimension(102, 32));
        btCancel.setSize(new java.awt.Dimension(102, 32));
        jPanel1.add(btCancel);

        jPanel2.setMinimumSize(new java.awt.Dimension(20, 32));
        jPanel2.setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.jPanel2.name")); // NOI18N
        jPanel2.setOpaque(false);
        jPanel2.setPreferredSize(new java.awt.Dimension(255, 32));
        jPanel2.setSize(new java.awt.Dimension(100, 32));

        pbMeasurementInProgress.setIndeterminate(true);
        pbMeasurementInProgress.setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.pbMeasurementInProgress.name")); // NOI18N
        pbMeasurementInProgress.setPreferredSize(new java.awt.Dimension(200, 20));
        jPanel2.add(pbMeasurementInProgress);

        lbInvitation.setForeground(new java.awt.Color(255, 51, 51));
        lbInvitation.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        lbInvitation.setText(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.lbInvitation.text")); // NOI18N
        lbInvitation.setName(org.openide.util.NbBundle.getMessage(ControlPanel.class, "ControlPanel.lbInvitation.name")); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                .addContainerGap())
            .addComponent(lbInvitation, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 107, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(lbInvitation, javax.swing.GroupLayout.DEFAULT_SIZE, 46, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 38, Short.MAX_VALUE)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 54, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btCancel;
    private javax.swing.JButton btContinue;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JLabel lbInvitation;
    private javax.swing.JProgressBar pbMeasurementInProgress;
    // End of variables declaration//GEN-END:variables
  }
